@@ -5,7 +5,7 @@
  * Copyright (C) 2005 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -23,9 +23,7 @@
  *          Hans Petter Jansson <hpj@novell.com>
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h> 
-#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,19 +36,24 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
-#include <libedataserver/e-dbhash.h>
-#include <libedataserver/e-db3-utils.h>
-#include <libedataserver/e-util.h>
-#include <libebook/e-contact.h>
 
-#include <libedata-book/e-book-backend-sexp.h>
-#include <libedata-book/e-book-backend-summary.h>
-#include <libedata-book/e-data-book.h>
-#include <libedata-book/e-data-book-view.h>
+#include "e-dbhash.h"
+#include "e-db3-utils.h"
+#include "libedataserver/e-util.h"
+
+#include "libebook/e-contact.h"
+
+#include "libedata-book/e-book-backend-sexp.h"
+#include "libedata-book/e-book-backend-summary.h"
+#include "libedata-book/e-data-book.h"
+#include "libedata-book/e-data-book-view.h"
+
 #include "e-book-backend-file.h"
 
-#define d(x) x
+#define d(x)
 
 #define CHANGES_DB_SUFFIX ".changes.db"
 
@@ -82,6 +85,24 @@ static struct {
 	DB_ENV *env;
 } global_env;
 
+static EDataBookStatus
+db_error_to_status (const int db_error)
+{
+	switch (db_error) {
+	case 0:
+		return Success;
+	case DB_NOTFOUND:
+		return ContactNotFound;
+	case EACCES:
+		return PermissionDenied;
+	case ENOMEM:
+	case ENOSPC:
+		return NoSpace;
+	default:
+		return OtherError;
+	}
+}
+
 static void
 string_to_dbt(const char *str, DBT *dbt)
 {
@@ -112,7 +133,7 @@ build_summary (EBookBackendFilePrivate *bfpriv)
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning ("build_summary: error building list\n");
+		g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 		return;
 	}
 
@@ -162,29 +183,33 @@ set_revision (EContact *contact)
 	
 }
 
-static EContact *
+static EDataBookStatus
 do_create(EBookBackendFile  *bf,
-	  const char      *vcard_req)
+	  const char      *vcard_req,
+	  EContact **contact)
 {
 	DB             *db = bf->priv->file_db;
 	DBT            id_dbt, vcard_dbt;
 	int            db_error;
 	char           *id;
-	EContact       *contact;
 	char           *vcard;
 	const char *rev;
 	
+	g_assert (bf);
+	g_assert (vcard_req);
+	g_assert (contact);
+
 	id = e_book_backend_file_create_unique_id ();
 
 	string_to_dbt (id, &id_dbt);
 
-	contact = e_contact_new_from_vcard (vcard_req);
-	e_contact_set (contact, E_CONTACT_UID, id);
-	rev = e_contact_get_const (contact,  E_CONTACT_REV);
+	*contact = e_contact_new_from_vcard (vcard_req);
+	e_contact_set (*contact, E_CONTACT_UID, id);
+	rev = e_contact_get_const (*contact,  E_CONTACT_REV);
 	if (!(rev && *rev))
-		set_revision (contact);
+		set_revision (*contact);
 
-	vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
+	vcard = e_vcard_to_string (E_VCARD (*contact), EVC_FORMAT_VCARD_30);
 
 	string_to_dbt (vcard, &vcard_dbt);
 
@@ -194,17 +219,17 @@ do_create(EBookBackendFile  *bf,
 
 	if (0 == db_error) {
 		db_error = db->sync (db, 0);
-		if (db_error != 0)
-			g_warning ("db->sync failed with %d", db_error);
-	}
-	else {
-		g_warning ("db->put failed with %d", db_error);
-		g_object_unref (contact);
-		contact = NULL;
+		if (db_error != 0) {
+			g_warning ("db->sync failed with %s", db_strerror (db_error));
+		}
+	} else {
+		g_warning (G_STRLOC ": db->put failed with %s", db_strerror (db_error));
+		g_object_unref (*contact);
+		*contact = NULL;
 	}
 
 	g_free (id);
-	return contact;
+	return db_error_to_status (db_error);
 }
 
 static EBookBackendSyncStatus
@@ -214,18 +239,14 @@ e_book_backend_file_create_contact (EBookBackendSync *backend,
 				    const char *vcard,
 				    EContact **contact)
 {
+	EDataBookStatus status;
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 
-	*contact = do_create (bf, vcard);
-	if (*contact) {
+	status = do_create (bf, vcard, contact);
+	if (status == Success) {
 		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
-		return GNOME_Evolution_Addressbook_Success;
-	}
-	else {
-		/* XXX need a different call status for this case, i
-                   think */
-		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
+	return status;
 }
 
 static EBookBackendSyncStatus
@@ -251,7 +272,8 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 
 		db_error = db->del (db, NULL, &id_dbt, 0);
 		if (0 != db_error) {
-			rv = GNOME_Evolution_Addressbook_ContactNotFound;
+			g_warning (G_STRLOC ": db->del failed with %s", db_strerror (db_error));
+			rv = db_error_to_status (db_error);
 			continue;
 		}
 
@@ -262,7 +284,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 	if (removed_cards) {
 		db_error = db->sync (db, 0);
 		if (db_error != 0)
-			g_warning ("db->sync failed.\n");
+			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
 	}
 
 	*ids = removed_cards;
@@ -312,10 +334,11 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 	/* get the old ecard - the one that's presently in the db */
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
-	if (0 != db_error)
+	if (0 != db_error) {
+		g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 		return GNOME_Evolution_Addressbook_ContactNotFound;
-
-	free (vcard_dbt.data);
+	}
+	g_free (vcard_dbt.data);
 
 	/* update the revisio (modified time of contact) */
 	set_revision (*contact);
@@ -327,19 +350,23 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 	if (0 == db_error) {
 		db_error = db->sync (db, 0);
-		if (db_error != 0)
-			g_warning ("db->sync failed.\n");
-
-		e_book_backend_summary_remove_contact (bf->priv->summary, id);
-		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
+		if (db_error != 0) {
+			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
+		} else {
+			e_book_backend_summary_remove_contact (bf->priv->summary, id);
+			e_book_backend_summary_add_contact (bf->priv->summary, *contact);
+		}
+	} else {
+		g_warning (G_STRLOC ": db->put failed with %s", db_strerror(db_error));
 	}
+
 	g_free (id);
 	g_free (vcard_with_rev);
 
 	if (0 == db_error)
 		return GNOME_Evolution_Addressbook_Success;
 	else
-		return GNOME_Evolution_Addressbook_ContactNotFound;
+		return db_error_to_status (db_error);
 }
 
 static EBookBackendSyncStatus
@@ -354,7 +381,7 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 	DBT             id_dbt, vcard_dbt;
 	int             db_error = 0;
 
-	bf = E_BOOK_BACKEND_FILE (e_data_book_get_backend (book));
+	bf = E_BOOK_BACKEND_FILE (backend);
 	db = bf->priv->file_db;
 
 	string_to_dbt (id, &id_dbt);
@@ -364,11 +391,10 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 	if (db_error == 0) {
-		*vcard = g_strdup (vcard_dbt.data);
-		printf ("backend %s\n", *vcard);
-		free (vcard_dbt.data);
+		*vcard = vcard_dbt.data;
 		return GNOME_Evolution_Addressbook_Success;
 	} else {
+		g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 		*vcard = g_strdup ("");
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
@@ -407,15 +433,15 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 			vcard_dbt.flags = DB_DBT_MALLOC;
 
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
-			if (db_error == 0)
-				contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
-			else { 
-				status = GNOME_Evolution_Addressbook_OtherError ;
+			if (db_error == 0) {
+				contact_list = g_list_prepend (contact_list, vcard_dbt.data);
+			} else {
+				g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
+				status = db_error_to_status (db_error);
 				break;
 			}
 		}
 		g_ptr_array_free (ids, TRUE);
-		
 	} else {
 		search_needed = TRUE;
 		if (!strcmp (search, "(contains \"x-evolution-any-field\" \"\")"))
@@ -424,17 +450,19 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		card_sexp = e_book_backend_sexp_new (search);
 		if (!card_sexp) {
 			/* XXX this needs to be an invalid query error of some sort*/
-			return GNOME_Evolution_Addressbook_ContactNotFound;
+			return GNOME_Evolution_Addressbook_OtherError;
 		}
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 
 		if (db_error != 0) {
+			g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 			/* XXX this needs to be some CouldNotOpen error */
-			return GNOME_Evolution_Addressbook_ContactNotFound;
+			return db_error_to_status (db_error);
 		}
 
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+		vcard_dbt.flags = DB_DBT_MALLOC;
 		memset (&id_dbt, 0, sizeof (id_dbt));
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
@@ -445,7 +473,7 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 
 				if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
-					contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
+					contact_list = g_list_prepend (contact_list, vcard_dbt.data);
 				}
 			}
 
@@ -454,13 +482,16 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		}
 		g_object_unref (card_sexp);
 		
-		status = db_error != DB_NOTFOUND
-			? GNOME_Evolution_Addressbook_OtherError
-			: GNOME_Evolution_Addressbook_Success;
+		if (db_error == DB_NOTFOUND) {
+			status = GNOME_Evolution_Addressbook_Success;
+		} else {
+			g_warning (G_STRLOC ": dbc->c_get failed with %s", db_strerror (db_error));
+			status = db_error_to_status (db_error);
+		}
 
 		db_error = dbc->c_close(dbc);
 		if (db_error != 0) {
-			g_warning("Could not close cursor: %d", db_error);
+			g_warning (G_STRLOC ": dbc->c_close failed with %s", db_strerror (db_error));
 		}
 	}
 
@@ -518,25 +549,28 @@ book_view_thread (gpointer data)
 	DB  *db;
 	DBT id_dbt, vcard_dbt;
 	int db_error;
-	gboolean stopped = FALSE;
+	gboolean stopped = FALSE, allcontacts;
 
 	d(printf ("starting initial population of book view\n"));
 
 	/* ref the book view because it'll be removed and unrefed
 	   when/if it's stopped */
-	bonobo_object_ref (book_view);
+	g_object_ref (book_view);
 	
 	db = bf->priv->file_db;
 	query = e_data_book_view_get_card_query (book_view);
 
-	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")"))
+	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")")) {
 		e_data_book_view_notify_status_message (book_view, _("Loading..."));
-	else
+		allcontacts = TRUE;
+	} else {
 		e_data_book_view_notify_status_message (book_view, _("Searching..."));
+		allcontacts = FALSE;
+	}
 
 	d(printf ("signalling parent thread\n"));
 	g_mutex_lock (closure->mutex);
-	g_cond_signal (closure->cond);
+	//g_cond_signal (closure->cond);
 	g_mutex_unlock (closure->mutex);
 
 	if (e_book_backend_summary_is_summary_query (bf->priv->summary, query)) {
@@ -560,16 +594,10 @@ book_view_thread (gpointer data)
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 			if (db_error == 0) {
-				EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
-				/* notify_update will check if it matches for us */
-				e_data_book_view_notify_update (book_view, contact);
-				
-				g_object_unref (contact);
-
-				free (vcard_dbt.data);
+				e_data_book_view_notify_update_prefiltered_vcard (book_view, id, vcard_dbt.data);
 			}
 			else {
-				g_warning ("db->get returned %d", db_error);
+				g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 			}
 		}
 
@@ -581,6 +609,7 @@ book_view_thread (gpointer data)
 
 		memset (&id_dbt, 0, sizeof (id_dbt));
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+		vcard_dbt.flags = DB_DBT_MALLOC;
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error == 0) {
@@ -597,10 +626,12 @@ book_view_thread (gpointer data)
 
 				/* don't include the version in the list of cards */
 				if (strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
-					EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
-					/* notify_update will check if it matches for us */
-					e_data_book_view_notify_update (book_view, contact);
-					g_object_unref (contact);
+					if (allcontacts)
+						e_data_book_view_notify_update_prefiltered_vcard (book_view, id_dbt.data, vcard_dbt.data);
+					else
+						e_data_book_view_notify_update_vcard (book_view, vcard_dbt.data);
+				} else {
+					g_free (vcard_dbt.data);
 				}
 
 				db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
@@ -623,8 +654,7 @@ book_view_thread (gpointer data)
 		e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_Success);
 
 	/* unref the */
-	printf("book_view file uref \n");
-	bonobo_object_unref (book_view);
+	g_object_unref (book_view);
 
 	d(printf ("finished population of book view\n"));
 
@@ -640,9 +670,9 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 	g_mutex_lock (closure->mutex);
 
 	d(printf ("starting book view thread\n"));
-	closure->thread = g_thread_create (book_view_thread, book_view, TRUE, NULL);
+	closure->thread = g_thread_create (book_view_thread, book_view, FALSE, NULL);
 
-	g_cond_wait (closure->cond, closure->mutex);
+	//g_cond_wait (closure->cond, closure->mutex);
 
 	/* at this point we know the book view thread is actually running */
 	g_mutex_unlock (closure->mutex);
@@ -654,17 +684,15 @@ e_book_backend_file_stop_book_view (EBookBackend  *backend,
 				    EDataBookView *book_view)
 {
 	FileBackendSearchClosure *closure = get_closure (book_view);
-	gboolean need_join = FALSE;
 
 	d(printf ("stopping query\n"));
+	if (!closure) {
+		/* TODO: find out why this happens! */
+		return;
+	}
 	g_mutex_lock (closure->mutex);
-	if (!closure->stopped)
-		need_join = TRUE;
 	closure->stopped = TRUE;
 	g_mutex_unlock (closure->mutex);
-
-	if (need_join)
-		g_thread_join (closure->thread);
 }
 
 typedef struct {
@@ -709,7 +737,7 @@ e_book_backend_file_changes_foreach_key (const char *key, gpointer user_data)
 
 		g_object_unref (contact);
 		
-		free (vcard_dbt.data);
+		g_free (vcard_dbt.data);
 	}
 }
 
@@ -747,7 +775,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning ("e_book_backend_file_changes: error building list\n");
+		g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 	} else {
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
@@ -798,7 +826,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	if (db_error != DB_NOTFOUND) {
 		g_warning ("e_book_backend_file_changes: error building list\n");
 		*changes_out = NULL;
-		result = GNOME_Evolution_Addressbook_OtherError;
+		result = db_error_to_status (db_error);
 	}
 	else {
 		/* Update the hash and build our changes list */
@@ -850,9 +878,9 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 static char *
 e_book_backend_file_extract_path_from_uri (const char *uri)
 {
-	g_assert (strncasecmp (uri, "file://", 7) == 0);
+	g_assert (g_ascii_strncasecmp (uri, "file://", 7) == 0);
 
-	return g_strdup (uri + 7);
+	return g_filename_from_uri (uri, NULL, NULL);
 }
 
 static EBookBackendSyncStatus
@@ -933,7 +961,7 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, char *old_version)
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error != 0) {
-			g_warning ("unable to get cursor");
+			g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 			return FALSE;
 		}
 
@@ -1010,8 +1038,7 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 	db_error = db->get (db, NULL, &version_name_dbt, &version_dbt, 0);
 	if (db_error == 0) {
 		/* success */
-		version = g_strdup (version_dbt.data);
-		free (version_dbt.data);
+		version = version_dbt.data;
 	}
 	else {
 		/* key was not in file */
@@ -1031,7 +1058,11 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 #endif
 
 static void
+#if DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3
+file_errcall (const DB_ENV *env, const char *buf1, const char *buf2)
+#else
 file_errcall (const char *buf1, char *buf2)
+#endif
 {
 	g_warning ("libdb error: %s", buf2);
 }
@@ -1060,10 +1091,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 	db_error = e_db3_utils_maybe_recover (filename);
 	if (db_error != 0) {
-		g_warning ("db recovery failed with %d", db_error);
+		g_warning ("db recovery failed with %s", db_strerror (db_error));
 		g_free (dirname);
 		g_free (filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		return db_error_to_status (db_error);
 	}
 
 	g_static_mutex_lock(&global_env_lock);
@@ -1073,25 +1104,30 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	} else {
 		db_error = db_env_create (&env, 0);
 		if (db_error != 0) {
-			g_warning ("db_env_create failed with %d", db_error);
+			g_warning ("db_env_create failed with %s", db_strerror (db_error));
 			g_static_mutex_unlock(&global_env_lock);
 			g_free (dirname);
 			g_free (filename);
-			return GNOME_Evolution_Addressbook_OtherError;
+			return db_error_to_status (db_error);
 		}
 
+		env->set_errcall (env, file_errcall);
+
+		/* Set the allocation routines to the non-aborting GLib functions */
+		env->set_alloc (env, (void *(*)(size_t))g_try_malloc, 
+				(void *(*)(void *, size_t))g_try_realloc,
+				g_free);
+
 		db_error = env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
 		if (db_error != 0) {
 			env->close(env, 0);
-			g_warning ("db_env_open failed with %d", db_error);
+			g_warning ("db_env_open failed with %s", db_strerror (db_error));
 			g_static_mutex_unlock(&global_env_lock);
 			g_free (dirname);
 			g_free (filename);
-			return GNOME_Evolution_Addressbook_OtherError;
+			return db_error_to_status (db_error);
 		}
 
-		env->set_errcall (env, file_errcall);
-
 		global_env.env = env;
 		global_env.ref_count = 1;
 	}
@@ -1101,10 +1137,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 	db_error = db_create (&db, env, 0);
 	if (db_error != 0) {
-		g_warning ("db_create failed with %d", db_error);
+		g_warning ("db_create failed with %s", db_strerror (db_error));
 		g_free (dirname);
 		g_free (filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		return db_error_to_status (db_error);
 	}
 
 	db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1113,10 +1149,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		db_error = e_db3_utils_upgrade_format (filename);
 
 		if (db_error != 0) {
-			g_warning ("db format upgrade failed with %d", db_error);
+			g_warning ("db format upgrade failed with %s", db_strerror (db_error));
 			g_free (dirname);
 			g_free (filename);
-			return GNOME_Evolution_Addressbook_OtherError;
+			return db_error_to_status (db_error);
 		}
 
 		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1140,22 +1176,25 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 				g_free (dirname);
 				g_free (filename);
 				if (errno == EACCES || errno == EPERM)
-					return GNOME_Evolution_Addressbook_PermissionDenied;
+					return PermissionDenied;
+				else if (errno == ENOSPC)
+					return NoSpace;
 				else
-					return GNOME_Evolution_Addressbook_OtherError;
+					return OtherError;
 			}
 
 			db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_CREATE | DB_THREAD, 0666);
 			if (db_error != 0) {
-				g_warning ("db->open (... DB_CREATE ...) failed with %d", db_error);
+				g_warning ("db->open (... DB_CREATE ...) failed with %s", db_strerror (db_error));
 			}
 			else {
 #ifdef CREATE_DEFAULT_VCARD
-				EContact *contact;
+				EContact *contact = NULL;
 
-				contact = do_create(bf, XIMIAN_VCARD);
+				do_create(bf, XIMIAN_VCARD, &contact);
 				/* XXX check errors here */
-				g_object_unref (contact);
+				if (contact)
+					g_object_unref (contact);
 #endif
 
 				writable = TRUE;
@@ -1167,7 +1206,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		bf->priv->file_db = NULL;
 		g_free (dirname);
 		g_free (filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		return db_error_to_status (db_error);
 	}
 
 	if (!e_book_backend_file_maybe_upgrade_db (bf)) {
@@ -1184,7 +1223,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	bf->priv->dirname = dirname;
 	bf->priv->filename = filename;
 
-	if (stat (bf->priv->filename, &sb) == -1) {
+	if (g_stat (bf->priv->filename, &sb) == -1) {
 		db->close (db, 0);
 		bf->priv->file_db = NULL;
 		g_warning ("stat(%s) failed", bf->priv->filename);
@@ -1232,7 +1271,7 @@ e_book_backend_file_remove (EBookBackendSync *backend,
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	GDir *dir;
 
-	if (-1 == unlink (bf->priv->filename)) {
+	if (-1 == g_unlink (bf->priv->filename)) {
 		if (errno == EACCES || errno == EPERM)
 			return GNOME_Evolution_Addressbook_PermissionDenied;
 		else
@@ -1242,7 +1281,7 @@ e_book_backend_file_remove (EBookBackendSync *backend,
 	/* unref the summary before we remove the file so it's not written out again */
 	g_object_unref (bf->priv->summary);
 	bf->priv->summary = NULL;
-	if (-1 == unlink (bf->priv->summary_filename))
+	if (-1 == g_unlink (bf->priv->summary_filename))
 		g_warning ("failed to remove summary file `%s`: %s", bf->priv->summary_filename, strerror (errno));
 
 	dir = g_dir_open (bf->priv->dirname, 0, NULL);
@@ -1252,7 +1291,7 @@ e_book_backend_file_remove (EBookBackendSync *backend,
 		while ((name = g_dir_read_name (dir))) {
 			if (select_changes (name)) {
 				char *full_path = g_build_filename (bf->priv->dirname, name, NULL);
-				if (-1 == unlink (full_path)) {
+				if (-1 == g_unlink (full_path)) {
 					g_warning ("failed to remove change db `%s': %s", full_path, strerror (errno));
 				}
 				g_free (full_path);
@@ -1262,7 +1301,7 @@ e_book_backend_file_remove (EBookBackendSync *backend,
 		g_dir_close (dir);
 	}
 
-	if (-1 == rmdir (bf->priv->dirname))
+	if (-1 == g_rmdir (bf->priv->dirname))
 		g_warning ("failed to remove directory `%s`: %s", bf->priv->dirname, strerror (errno));
 
 	/* we may not have actually succeeded in removing the
@@ -1285,13 +1324,29 @@ e_book_backend_file_cancel_operation (EBookBackend *backend, EDataBook *book)
 	return GNOME_Evolution_Addressbook_CouldNotCancel;
 }
 static void 
-e_book_backend_file_set_mode (EBookBackend *backend,  GNOME_Evolution_Addressbook_BookMode mode)
+e_book_backend_file_set_mode (EBookBackend *backend,  EDataBookMode mode)
 {
 	if (e_book_backend_is_loaded (backend)) {
 		e_book_backend_notify_writable (backend, TRUE);
 		e_book_backend_notify_connection_status (backend, TRUE);
 	}
 }
+
+static void 
+e_book_backend_file_sync (EBookBackend *backend)
+{
+	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
+	int db_error;
+	
+	g_return_if_fail (bf != NULL);
+	
+	if (bf->priv->file_db) {
+		db_error = bf->priv->file_db->sync (bf->priv->file_db, 0);
+		if (db_error != 0)
+			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
+	}
+}
+
 static gboolean
 e_book_backend_file_construct (EBookBackendFile *backend)
 {
@@ -1355,6 +1410,50 @@ e_book_backend_file_dispose (GObject *object)
 	G_OBJECT_CLASS (e_book_backend_file_parent_class)->dispose (object);	
 }
 
+#ifdef G_OS_WIN32
+/* Avoid compiler warning by providing a function with exactly the
+ * prototype that db_env_set_func_open() wants for the open method.
+ */
+
+static int
+my_open (const char *name, int oflag, ...)
+{
+	int mode = 0;
+
+	if (oflag & O_CREAT) {
+		va_list arg;
+		va_start (arg, oflag);
+		mode = va_arg (arg, int);
+		va_end (arg);
+	}
+
+	return g_open (name, oflag, mode);
+}
+
+int
+my_rename (const char *oldname, const char *newname)
+{
+	return g_rename (oldname, newname);
+}
+
+int
+my_exists (const char *name, int *isdirp)
+{
+	if (!g_file_test (name, G_FILE_TEST_EXISTS))
+		return ENOENT;
+	if (isdirp != NULL)
+		*isdirp = g_file_test (name, G_FILE_TEST_IS_DIR);
+	return 0;
+}
+
+int
+my_unlink (const char *name)
+{
+	return g_unlink (name);
+}
+
+#endif
+
 static void
 e_book_backend_file_class_init (EBookBackendFileClass *klass)
 {
@@ -1374,6 +1473,7 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	backend_class->stop_book_view          = e_book_backend_file_stop_book_view;
 	backend_class->cancel_operation        = e_book_backend_file_cancel_operation;
 	backend_class->set_mode                = e_book_backend_file_set_mode;
+	backend_class->sync                    = e_book_backend_file_sync;
 	sync_class->remove_sync                = e_book_backend_file_remove;
 	sync_class->create_contact_sync        = e_book_backend_file_create_contact;
 	sync_class->remove_contacts_sync       = e_book_backend_file_remove_contacts;
@@ -1387,6 +1487,17 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	
 
 	object_class->dispose = e_book_backend_file_dispose;
+
+#ifdef G_OS_WIN32
+	/* Use the gstdio wrappers to open, check, rename and unlink
+	 * files from libdb.
+	 */
+	db_env_set_func_open (my_open);
+	db_env_set_func_close (close);
+	db_env_set_func_exists (my_exists);
+	db_env_set_func_rename (my_rename);
+	db_env_set_func_unlink (my_unlink);
+#endif
 }
 
 static void
